#  A simple interface to search the MITSFS pinkdex.
#  Calvin Clark <ckclark@mit.edu>
# 
#  $Id: pinkdex.pl,v 1.3 1995/04/07 05:49:40 ckclark Exp $

# $map{'pinkdex'} = '&do_pinkdex ($query)';

package pinkdex;

$pinkdex_file = "/afs/athena.mit.edu/activity/m/mitsfs/Public/pinkdex.txt";

$title_checked = "CHECKED";
$author_cehcked = "";
$last_search = "";


sub info {
    if ($ENV{'SERVER_PROTOCOL'} eq "HTTP/1.0") {
	print <<"EndOfInfo";
<!doctype html public "-//W3O//DTD W3 HTML 3.0//EN">
<HTML>
<HEAD>
<TITLE>MITSFS Library Pinkdex</TITLE>
<BASE HREF="http://www.mit.edu:8001/pinkdex">
</HEAD>
<BODY>
<!-- Response for $ENV{'SERVER_PROTOCOL'} query -->
<h1>MITSFS Library Pinkdex</h1>
<p>
This is a simple interface to search the
<a href="http://www.mit.edu:8001/activities/mitsfs/homepage.html"> 
MIT Science Fiction Library's</a>
catalog.  You can search by author or by title.  Matches are case
insensitive.  Bug reports and comments should be e-mailed to
<tt><a href="http://www.mit.edu:8001/people/ckclark/home.html">ckclark@mit.edu</a></tt>.
<p>
Examples: <p>
<ul>
<li> An author search: <p>
<code>zelazny</code><p>
<li> A title search:  Don't try to quote any spaces or characters; everything
after title or author is passed as a single token. <p>
<code>foundation and empire</code><p>
<li> A more specific author search: <p>
<code>adams, douglas</code><p>
<li> A regular expression title search: <p>
<code>star trek [IVX]+</code><p>
<p>
</ul>
EndOfInfo
    } else {
	print <<"EndOfOldInfo";
<!doctype html public "-//W3O//DTD W3 HTML 3.0//EN">
<HTML>
<HEAD>
<TITLE>MITSFS Library Pinkdex</TITLE>
<BASE HREF="http://www.mit.edu:8001/pinkdex">
<ISINDEX>
</HEAD>
<BODY>
<!-- Response for $ENV{'SERVER_PROTOCOL'} query -->
<h1>MITSFS Library Pinkdex</h1>
<p>
This is a simple interface to search the 
<a href="http://www.mit.edu:8001/mitsfs.html">
MIT Science Fiction Library's</a>
catalog.  (A somewhat better interface using HTTP 1.0 forms
would be available to you if you were using a HTTP 1.0 or later client
but you're not.)  Bug reports and comments
should be e-mailed to:
<tt><a href="http://www.mit.edu:8001/people/ckclark/home.html">ckclark@mit.edu</a></tt>.
<p>
You can search by author or by title.  Matches are case
insensitive.
<p>
Examples: <p>
<ul>
<li> An author search: <p>
<code>author zelazny</code><p>
<li> A title search:  Don't try to quote any spaces or characters; everything
after title or author is passed as a single token. <p>
<code>title foundation and empire</code><p>
<li> A more specific author search: <p>
<code>author adams, douglas</code><p>
<li> A regular expression title search: <p>
<code>title star trek [IVX]+</code><p>
<p>
</ul>
EndOfOldInfo
    }
	if ($ENV{'SERVER_PROTOCOL'} eq "HTTP/1.0") {
		print <<"EOF";
<FORM ACTION="http://www.mit.edu:8001/pinkdex">
<b>Search Type:</b><p>
Title Search <INPUT TYPE="radio" NAME="type" VALUE="title" $title_checked>
Author Search <INPUT TYPE="radio" NAME="type" VALUE="author" $author_checked>
<p>
<b>Search for:</b><INPUT NAME="regexp" SIZE=80 VALUE="$last_search">
<INPUT TYPE="submit" VALUE="Do Search">
</BODY>
</HTML>
EOF
	} else {
		print "</BODY>\n</HTML>\n";
	}
}


sub main'do_pinkdex {
    local ($query) = @_;
    local (*PINKDEX);
    local ($type, $regexp, $type_keyword, $regexp_keyword);
    local ($match, $author, $title, $series);
    local ($regexp1, $regexp2);

    $query =~ y/\+/ /;

    if (! $query) {
	&info;
	return;
    }
  
    if ($ENV{'SERVER_PROTOCOL'} eq "HTTP/1.0") {
	($type_keyword, $type, $regexp_keyword, $regexp) = split (/[=&]/, $query);
        if ($type_keyword ne "type" || $regexp_keyword ne "regexp") {
	   &error("bogon query values for FORM");
        }
        $regexp =~ s/%([\da-f]{1,2})/pack(C,hex($1))/eig; # remove % escapes
	if ($type eq "author") {
		$author_checked = "CHECKED";
		$title_checked = "";
	} else {
		$author_checked = "";
		$title_checked = "CHECKED";
	}
    } else {
    	($type, $regexp) = split (/ +/, $query, 2);
    }
    eval { /$regexp/ };
    if ($@ ne "") {
	&error("error in regular expression: $@");
    }

    $type =~ y/A-Z/a-z/;
    $last_search = $regexp;

    if ($type eq "author") {
	$regexp1 = ".*$regexp.*";
	$regexp2 = ".*";
    }
    else { 
	if ($type eq "title") {
	    $regexp1 = ".*";
	    $regexp2 = ".*$regexp.*";
	} 
	else {
	    &error ('you must specify author or title search');
	}
    }

    open (PINKDEX, "<$pinkdex_file") 
	|| &error ("error opening $pinkdex_file: $!");
    if ($ENV{'SERVER_PROTOCOL'} eq "HTTP/1.0") {
      print <<"EOH"
<!doctype html public "-//W3O//DTD W3 HTML 3.0//EN">
<HTML>
<HEAD>
<TITLE>MITSFS Pinkdex search results</TITLE>
</HEAD>
<BODY>
EOH
    } else {
      print <<"EOH"
<!doctype html public "-//W3O//DTD W3 HTML 3.0//EN">
<HTML>
<HEAD>
<TITLE>MITSFS Pinkdex search results</TITLE>
<ISINDEX>
</HEAD>
EOH
    }

    print '<a href="http://www.mit.edu:8001/pinkdex"><i>back to pinkdex help</i></a><p>';
    print "<h1>results of $type search for: $regexp</h1><p>\n";
    if ($ENV{'SERVER_PROTOCOL'} eq "HTTP/1.0") {
	print <<"EOF";
<FORM ACTION="http://www.mit.edu:8001/pinkdex">
<b>Search Type:</b><p>
Title Search <INPUT TYPE="radio" NAME="type" VALUE="title" $title_checked>
Author Search <INPUT TYPE="radio" NAME="type" VALUE="author" $author_checked>
<p>
<b>Search for:</b><INPUT NAME="regexp" SIZE=80 VALUE="$last_search">
<INPUT TYPE="submit" VALUE="Do Search">
EOF
    }
    print "<p>\n";
    while (<PINKDEX>) {
	chop;
	$line = $_;
	if (/^($regexp1)<($regexp2)<(.*)<(.*)/oi) {
	    if ($match ne $1) {
		$match = $1;
		print "<h2>$match:</h2>\n";
	    }
	    $title = $2;
	    $series = ($3 ? " [$3]" : "");
	    $line =~ s/([\000-\044+\177-\377])/sprintf('%%%02x',ord($1))/eg;
	    print "\t<a href=\"http://www.mit.edu:8001/pinkdex_v?$line\">$title$series</a><br>\n";
	}
    }
    if (! $match) {
	print "<b>No matches found.</b><p>\n";
    }

    print "\n</BODY>\n</HTML>";
    close (PINKDEX);
}

%shelf_codes = (
  "A", "Art Book",
  "B", "Booklet",
  "BLB", "Big (or Better) Little Book",
  "BS", "Binary Star #",
  "C/BS", "Circulating Binary Star #",
  "C/D", "Circulating Ace Double #",
  "C/P", "Circulating Paperback",
  "C/PA", "Circulating Paperback Anthology",
  "CAL", "Calendar",
  "CH", "Cabinet Hardcover",
  "CP", "Cabinet Paperback",
  "CX", "Comics",
  "D", "Ace Double #",
  "FH", "Foreign Hardcover",
  "FP", "Foreign Paperback",
  "GN", "Galaxy Novel #",
  "H", "Hardcover",
  "HA", "Hardcover Anthology",
  "I", "Index",
  "IF", "Index Folder",
  "IL", "Locked Index",
  "LP", "Large Paperback",
  "P", "Paperback",
  "PA", "Paperback Anthology",
  "REF", "Dictionary",
  "RH", "Random Hardcover",
  "RP", "Random Paperback",
  "SCX", "Small Comics",
  "SFWA", "SFWA Hardcover",
  "SFWA-P", "SFWA Paperback",
  "SFWA-TD", "SFWA Tor Double",
  "SFWA-VLP", "SFWA Very Large Paperback",
  "SR-D", "Special Reserve Ace Double",
  "SR-DNESFA", "Special Reserve NESFA Double",
  "SR-H", "Special Reserve Hardcover",
  "SR-HA", "Special Reserve Hardcover Anthology",
  "SR-L", "Special Reserve Locked Cabinet",
  "SR-LP", "Special Reserve Large Paperback",
  "SR-MM", "Special Reserve Mixed Media",
  "SR-NESFA", "Special Reserve NESFA",
  "SR-P", "Special Reserve Paperback",
  "SR-PA", "Special Reserve Paperback Anthology",
  "SR-S", "Special Reserve Script",
  "SR-T", "Special Reserve Tape",
  "TD", "Tor Double",
  "VLP", "Very Large Paperback",
  "WRD", "Weird Categories",
);

sub parse_shelf_code {
	local ($rawcode) = @_;
	local (@entries) = split (",", $rawcode);

	print <<EOT;
<h2>shelf code description</h2><p>
raw shelf code: $rawcode<p><pre>
EOT
	for (@entries) {
		# delete trailing :'s and numbers
		s/[:0-9]+//;
		local ($description) = $shelf_codes{$_};
		if ($description) {
			printf ("%10s: %s\n", $_, $description);
		} else {
			printf ("%10s: UNKNOWN CODE!\n", $_);		
		}
	}
	print "</pre>\n";
	print "<a href=\"http://www.mit.edu:8001/Pinkdex_shelfcodes\">list of possible shelf codes\n</a><p>";
}

sub main'do_pinkdex_verbose {
	local ($query) = @_;

	$query =~ s/%([\da-f]{1,2})/pack(C,hex($1))/eig; # remove % escapes

	local ($author, $title, $series, $shelfcode) = split (/</, $query);
	$series = ($series ? " [$series]" : "");
	print <<"EOH";
<!doctype html public "-//W3O//DTD W3 HTML 3.0//EN">
<HTML>
<HEAD>
<TITLE>MITSFS Library Pinkdex: $title$series</TITLE>
</HEAD>
<BODY>
<!-- Response for $ENV{'SERVER_PROTOCOL'} query -->
EOH
	print "<h1>Title: $title$series</h1>\n";
	print "<h1>Author(s): $author</h1>\n";
	$author =~ s/([\000-\044+\177-\377])/sprintf('%%%02x',ord($1))/eg;
	print "<a href=\"http://www.mit.edu:8001/pinkdex?type=author&regexp=$author\">Select to retrieve other titles by this author</a><p>\n";
	&parse_shelf_code ($shelfcode);
	print "<a href=\"http://www.mit.edu:8001/pinkdex\"><i>back to pinkdex help</i></a><p>\n</BODY></HTML>";
}

sub error {
    local($msg) = @_;
    print  <<EOM
<!doctype html public "-//W3O//DTD W3 HTML 3.0//EN">
<HTML>
<HEAD>
<TITLE>Server Error: 400</TITLE>
</HEAD>
<BODY>
<H1>Server Error: 400</H1>
<STRONG>$msg</STRONG> <P>
If you feel this is a server problem and wish to report it, please
include the error code, the requested URL, which and what version
browser you are using, and any other facts that might be relevant to: <P>
<a href="http://www.mit.edu:8001/comment">webmaster@mit.edu</a>
</BODY>
</HTML>
EOM
}

1;
